# Copyright 2018 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import absolute_import
import subprocess
import os
from netq_lib.orm.redisdb.models import Port

local_port_dict = dict()


def process_ports(output, svc_failed=False, **kwargs):
    fnull = open(os.devnull, 'w')
    port_names = kwargs.get('ports', None)
    platform = kwargs.get('platform', '')
    last_ports = set(local_port_dict.keys())
    curr_ports = port_names
    del_ports = last_ports - curr_ports
    # handle ports disappear, e.g., split ports
    if del_ports:
        for dport_name in del_ports:
            port = Port()
            port.ifname = dport_name
            port.delete()
            del local_port_dict[dport_name]

    if not port_names:
        return
    for port_name in port_names:
        # On some platforms, due to slow I2C reads or related issues, we are
        # to read the port EEPROMs leading us to reach the false conclusion
        # that the port is empty. To avoid such issues, we save data only if
        # see the same data twice. "changes" keep track of that. Only if there
        # is no change do we try to "save" data.
        change = False
        # ignore PIM interfaces and 'swid0_eth' seen on MLX boxes.
        if (port_name.startswith('pimreg') or port_name == 'swid0_eth' or
                port_name.startswith('usb')):
            continue
        if port_name not in local_port_dict.keys():
            local_port_dict[port_name] = dict()
            local_port_dict[port_name]['identifier'] = 'n/a'
            local_port_dict[port_name]['test_serial_number'] = 'n/a'
            local_port_dict[port_name]['vendor_name'] = 'n/a'
            local_port_dict[port_name]['serial_number'] = 'n/a'
            local_port_dict[port_name]['part_number'] = 'n/a'
            local_port_dict[port_name]['transreceiver'] = 'n/a'
            local_port_dict[port_name]['connector'] = 'n/a'
            local_port_dict[port_name]['length'] = 'n/a'

        ethtool_dict = dict()
        try:
            ethtool_dump = subprocess.check_output(['/sbin/ethtool', port_name],
                                                   stderr=fnull)
            for line in ethtool_dump.splitlines():
                if ':' not in line:
                    continue
                key, value = line.split(':', 1)
                key = key.strip()
                value = value.strip()
                if key in ethtool_dict.keys():
                    ethtool_dict[key] = ethtool_dict[key] + '| ' + value
                else:
                    ethtool_dict[key] = value
        except subprocess.CalledProcessError:
            pass

        try:
            ethtool_dump = subprocess.check_output(['/sbin/ethtool', '--show-fec', port_name],
                                                   stderr=fnull)
            for line in ethtool_dump.splitlines():
                if ":" not in line:
                    continue
            key, value = line.split(':', 1)
            key = key.strip()
            value = value.strip()
            if key in ethtool_dict.keys():
                ethtool_dict[key] = ethtool_dict[key] + '| ' + value
            else:
                ethtool_dict[key] = value
        except subprocess.CalledProcessError:
            pass


        port = Port()
        port.ifname = port_name
        eth_port = ethtool_dict.get('Port', 'n/a')
        eth_supp_port = ethtool_dict.get('Supported ports', 'n/a')
        identifier = 'empty'
        if eth_port.startswith('Twisted') and 'tp' in eth_supp_port.lower():
            identifier = 'RJ45'
        elif 'vx' in platform:
            identifier = 'RJ45'
        elif 'backplane' in eth_supp_port.lower():
            identifier = 'backplane'

        speed = ethtool_dict.get('Speed', 'n/a').strip('!')
        if 'Mb/s' in speed:
            try:
                speed = '%dG' % (int(speed.split('M', 1)[0]) / 1000)
            except ValueError:
                pass
        port.speed = speed

        port.autoneg = ethtool_dict.get('Auto-negotiation', 'n/a')
        port.state = ethtool_dict.get('Link detected', 'n/a')
        if port.state == 'yes':
            port.state = 'up'
        elif port.state == 'no':
            port.state = 'down'
        port.fec = ethtool_dict.get('FEC encodings', 'n/a')
        port.supported_fec = ethtool_dict.get('Supported FEC modes', 'n/a')
        port.advertised_fec = ethtool_dict.get('Advertised FEC modes', 'n/a')

        ethtool_m_dict = dict()
        try:
            test_serial_number = subprocess.check_output(
                ['/sbin/ethtool', '-m', port_name, 'offset', '168', 'length',
                 '16', 'raw', 'on'], stderr=fnull)
        except subprocess.CalledProcessError:
            test_serial_number = None
        if test_serial_number != local_port_dict[port_name]['test_serial_number']:
            local_port_dict[port_name]['test_serial_number'] = test_serial_number
            change = True
            if not test_serial_number:
                local_port_dict[port_name]['identifier'] = identifier
                local_port_dict[port_name]['vendor_name'] = 'n/a'
                local_port_dict[port_name]['serial_number'] = 'n/a'
                local_port_dict[port_name]['part_number'] = 'n/a'
                local_port_dict[port_name]['transreceiver'] = 'n/a'
                local_port_dict[port_name]['connector'] = 'n/a'
                local_port_dict[port_name]['length'] = 'n/a'
            else:
                try:
                    ethtool_m_dump = subprocess.check_output(
                        ['/sbin/ethtool', '-m', port_name], stderr=fnull)
                    for line in ethtool_m_dump.splitlines():
                        key, value = line.split(':', 1)
                        key = key.strip()
                        value = value.strip()
                        if key.startswith('Length'):
                            key = 'length'
                        value = value.strip()
                        if key == 'length' and value == '0m' or value == '0km':
                            continue
                        if key in ethtool_m_dict.keys():
                            ethtool_m_dict[key] = ethtool_m_dict[key] + '| ' + value
                        else:
                            ethtool_m_dict[key] = value
                    ident = ethtool_m_dict.get('Identifier', 'n/a').strip()
                    if ('no' in ident.lower() or
                            (identifier != 'RJ45' and ident == 'n/a')):
                        ident = 'empty'
                    if '(' in ident:
                        ident = ident.replace('(', '').replace(')', '')
                        p_id, p_type = ident.strip().split(' ', 1)
                        ident = p_type
                    local_port_dict[port_name]['identifier'] = ident
                    local_port_dict[port_name]['serial_number'] = ethtool_m_dict.get('Vendor SN', 'n/a')
                    local_port_dict[port_name]['vendor_name'] = ethtool_m_dict.get('Vendor name', 'n/a')
                    local_port_dict[port_name]['part_number'] = ethtool_m_dict.get('Vendor PN', 'n/a')
                    local_port_dict[port_name]['length'] = ethtool_m_dict.get('length', 'n/a').replace('|', ',')
                    trans = ethtool_m_dict.get('Transceiver type', 'n/a')
                    trans_type = set()
                    for x in trans.strip().split('|'):
                        if 'FC:' in x:
                            if 'MBytes/sec' not in x:
                                y = x.split(':')[1].strip()
                                trans_type.add(y.strip())
                        elif 'Ethernet:' in x:
                            y = x.split(':')[1].strip()
                            y = y.replace('BASE-', 'Base-')
                            trans_type.add(y.strip())
                        elif ('SONET' in x or 'Infiniband' in x or
                              'short reach' in x or 'M5' in x):
                            continue
                        else:
                            blacklist = ['SONET', 'Infiniband', 'short reach',
                                         'M5', 'OM3']
                            found = False
                            for b in blacklist:
                                if b in x:
                                    found = True
                            if not found:
                                trans_type.add(x.strip())
                    local_port_dict[port_name]['transreceiver'] = ','.join(
                        sorted(trans_type))

                    conn = ethtool_m_dict.get('Connector', 'n/a').strip()
                    if ' ' in conn:
                        conn = conn.split(' ', 1)[1].replace(
                            '(', '').replace(')', '')
                        if conn == 'No separable connector':
                            conn = 'n/a'
                        elif conn == 'MPO Parallel Optic':
                            conn = 'MPO Parallel'

                    local_port_dict[port_name]['connector'] = conn
                except subprocess.CalledProcessError:
                    pass

        port.identifier = local_port_dict[port_name]['identifier']
        port.vendor_name = local_port_dict[port_name]['vendor_name']
        port.serial_number = local_port_dict[port_name]['serial_number']
        port.part_number = local_port_dict[port_name]['part_number']
        port.transreceiver = local_port_dict[port_name]['transreceiver']
        port.connector = local_port_dict[port_name]['connector']
        port.length = local_port_dict[port_name]['length']
        if not change:
            port.save()
