# Copyright 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import absolute_import

from netq_lib.orm.redisdb.models import ProcNetDevInterface


def process_proc_net_dev(output, svc_failed=False, **kwargs):
    """
    Parse /proc/net/dev counters
    """


    with open('/proc/net/dev', 'r') as fh:
        '''
        /proc/net/dev has the following format

Inter-|   Receive                                                |  Transmit
 face |bytes    packets errs drop fifo frame compressed multicast|bytes    packets errs drop fifo colls carrier compressed
 swp27:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp19:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp50:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp42:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp10:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp34:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
  swp7: 14386332   36183    0 4459    0     0          0     13319 62337842   81001    0  454    0     0       0          0
 swp26:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
        '''
        for line in fh.readlines()[2:]:
            (ifname,
             rx_bytes, rx_packets, rx_errs, rx_drop, rx_fifo, rx_frame, rx_compressed, rx_multicast,
             tx_bytes, tx_packets, tx_errs, tx_drop, tx_fifo, tx_colls, tx_carrier, tx_compressed) = line.strip().split()

            # chop the trailing :
            ifname = ifname[0:-1]

            # This is a virtual interface that is created as part of VRR, ignore it
            if ifname.endswith('-v0'):
                continue

            # fifo and compressed related counters are intentionally left out
            iface = ProcNetDevInterface()
            iface.ifname = ifname
            iface.rx_bytes = rx_bytes
            iface.rx_drop = rx_drop
            iface.rx_errs = rx_errs
            iface.rx_frame = rx_frame
            iface.rx_multicast = rx_multicast
            iface.rx_packets = rx_packets
            iface.tx_bytes = tx_bytes
            iface.tx_carrier = tx_carrier
            iface.tx_colls = tx_colls
            iface.tx_drop = tx_drop
            iface.tx_errs = tx_errs
            iface.tx_packets = tx_packets
            iface.save()
